# app.py
import streamlit as st
import requests
import asyncio
import aiohttp
import pandas as pd
import numpy as np
import networkx as nx
import plotly.express as px
from plotly.subplots import make_subplots
import plotly.graph_objects as go
import matplotlib.pyplot as plt
import io
from streamlit_option_menu import option_menu
import seaborn as sns


async def main():
    st.set_page_config(page_title="Uslugi santehnika", page_icon="🤖")
    with open('style.css') as f:
        st.markdown(f'<style>{f.read()}</style>', unsafe_allow_html=True)

    st.title("Виртуальный датчик")
    
    async with aiohttp.ClientSession() as session:

        # Sidebar Menu
        with st.sidebar:
            selected = option_menu(
                "Меню", ["Загрузка файлов", "Анализ данных"],
                icons=["house", "graph-up-arrow"], menu_icon="cast", default_index=0)

        uploaded_file = st.file_uploader("Выберите файл с признаками", type=["csv"], key="my-upload-button")
        normal_column_names = telemetry_dict = {
            'telemetry_0': 'temp4',  #Температура (датчик 4)
            'telemetry_1': 'consumption',  #Питание (загрузка печи)
            'telemetry_2': 'temp1',  #Температура (датчик 1)
            'telemetry_3': 'ampere_add',  #Сила тока
            'telemetry_4': 'press_add',  #Давление (датчик 3)
            'telemetry_5': 'temp3',  #Температура (датчик 3)
            'telemetry_6': 'speed',  #Скорость
            'telemetry_7': 'press1',  #Давление (датчик 1)
            'telemetry_8': 'press2',  #Давление (датчик 2)
            'telemetry_9': 'temp2',  #Температура (датчик 2)
            'telemetry_10': 'temp5',  #Температура (датчик 5)
            'telemetry_11': 'temp6',  #Температура (датчик 6)
            'telemetry_12': 'item1',  #Концентрация газа 1
            'telemetry_13': 'item2',  #Концентрация газа 2
            'telemetry_14': 'item3',  #Концентрация газа 3
            'telemetry_15': 'item4'  #Концентрация газа 4
        }
        # generate_file_input = st.button("Иллюстрация графиков")
        if selected == "Загрузка файлов":
            generate_file_input = st.button("Иллюстрация графиков")
            if generate_file_input and uploaded_file:
                
                files = {"file": (uploaded_file.name, uploaded_file.getvalue())}
                response = requests.post("http://localhost:8000/uploadfile/", files=files)
                if response.status_code == 200:
                    s = response.json().get('content')

                    df1 = pd.read_csv(io.StringIO(s), sep=",")
                

                df1.rename(columns=normal_column_names, inplace=True)
                df1['datetime'] = pd.to_datetime(df1['datetime'])
                df1.drop(columns=['item1', 'item2', 'item3', 'item4'], inplace=True)
                # fig = px.line(df1, y='target', title='График распределения предикт', labels={'index': 'Дата', 'target': 'Значение'})
                # st.plotly_chart(fig)
                st.write(df1)
            
            uploaded_train_file = st.file_uploader("Выберите файл target", type=["csv"])
            generate_train_file_input = st.button("Иллюстрация графиков target")

            if generate_train_file_input and uploaded_train_file:
                df = pd.read_csv(uploaded_train_file)
                df.set_index(df.columns[0], inplace=True)
                fig = px.line(df, y='target', title='График распределения target', labels={'index': 'Дата', 'target': 'Значение'})
                st.plotly_chart(fig)
                st.write(df)

        elif selected == "Анализ данных":
            st.header("Анализ данных")

            if uploaded_file:
                df = pd.read_csv(uploaded_file)
                df.rename(columns=normal_column_names, inplace=True)
                df['datetime'] = pd.to_datetime(df['datetime'])
                df.drop(columns=['item1', 'item2', 'item3', 'item4'], inplace=True)
                st.write(df)

                show_test_of_time = st.button("График показаний каждого параметра от времени")
                show_correlation_chart = st.button("График корреляции")

                if show_test_of_time:
                    fig, axs = plt.subplots(len(df.columns[1:]), 1, figsize=(10, 30), sharex=True)
                    for i, column in enumerate(sorted(df.columns[1:])):
                        axs[i].plot(pd.to_datetime(df['datetime']), df[column], label=column, color='blue')
                        axs[i].set_title(column)
                        axs[i].grid(True)
                        axs[i].legend()
                    fig.tight_layout()
                    plt.xlabel('Дата')
                    st.pyplot(fig)

                if show_correlation_chart:
                    # Удаление столбца с датами из данных для корреляции
                    df_corr = df.drop(columns=['datetime'])

                    # Построение корреляционной матрицы
                    corr_matrix = df_corr.corr()

                    # Отображение тепловой карты корреляционной матрицы
                    fig, ax = plt.subplots(figsize=(16, 14))
                    sns.heatmap(corr_matrix, annot=True, fmt=".2f", cmap="coolwarm", ax=ax)
                    ax.set_title("Корреляционная матрица", fontsize=16)

                    # Настройка меток оси x
                    ax.tick_params(axis='x', rotation=0)
                    ax.tick_params(axis='y', rotation=0)


                    st.pyplot(fig)




if __name__ == '__main__':
    asyncio.run(main())
